/*******************************************************************************
 * PVGA2/wd90c00/wd90c90 I/O - (extern variables and function headers)
 *******************************************************************************
 *
 *   Copyright (C) 1997 by Brian S. Julin, <bri@calyx.com>
 *
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *******************************************************************************
 *
 * Revision 0.1 
 * Major: Copied from IBM vga_io.inc in GGI 0.0.9
 *
 *******************************************************************************
 *	This code was derived from the following source of information:
 */

#ifndef _wd90c00_io_h
#define _wd90c00_io_h

#ifndef _wd90c2x_io_h  /* Flat panel control registers */
#ifndef _wd90c3x_io_h  /* Undocumented vertical timing register */
#define NoCRTRegs 0x31
#endif /* not _wd90c2x_io_h */
#endif /* not _wd90c3x_io_h */

#include <WD/pvga1a_io.h>
#include <WD/wd90c00.h>

static inline int wunlockCRT(void) 
{ 
  int lockval1, lockval2;
  lockval1 = rwunlockGRC();  /* Some CRT locks kept in upper GRC,
                               which can in turn be locked -- pain */
  lockval2 = inbGRC(GRC_CRTC_CTL);
  outbGRC((lockval2 | GR0D_CR11_NOLOCK /* Disables traditional VGA lock */)
	  & ~(GR0D_VTIMING_LOCK | GR0D_HTIMING_LOCK), GRC_CRTC_CTL);
  outbGRC(lockval1, GRC_UPGRC_LOCK); /* Restore locking of upper GRC */
  return ((outbitsCRT(CR29_WUNLOCK, CR29_WLOCK_MASK, CRT_UPCRT_LOCK) << 8) 
    | lockval2); /* Two locks to keep track of */
}

static inline int wlockCRT(void) 
{ 
  int lockval1, lockval2;
  lockval1 = rwunlockGRC();  /* Some CRT locks kept in upper GRC,
                               which can in turn be locked -- pain */
  lockval2 = inbGRC(GRC_CRTC_CTL);
  outbGRC((lockval2 | GR0D_CR11_NOLOCK /* Disables traditional VGA lock */
	  | GR0D_VTIMING_LOCK | GR0D_HTIMING_LOCK), GRC_CRTC_CTL);
  outbGRC(lockval1, GRC_UPGRC_LOCK); /* Restore locking of upper GRC */
  return ((outbitsCRT(CR29_WLOCK, CR29_WLOCK_MASK, CRT_UPCRT_LOCK) << 8) 
    | lockval2); /* Two locks to keep track of */
}

static inline int rwunlockCRT(void) 
{ 
  int lockval1, lockval2;
  lockval1 = rwunlockGRC();  /* Some CRT locks kept in upper GRC,
                               which can in turn be locked -- pain */
  lockval2 = inbGRC(GRC_CRTC_CTL);
  outbGRC((lockval2 | GR0D_CR11_NOLOCK /* Disables traditional VGA lock */)
	  & ~(GR0D_VTIMING_LOCK | GR0D_HTIMING_LOCK), GRC_CRTC_CTL);
  outbGRC(lockval1, GRC_UPGRC_LOCK); /* Restore locking of upper GRC */
  return ((outbitsCRT(CR29_WUNLOCK | CR29_RUNLOCK, 
		      CR29_WLOCK_MASK | CR29_RLOCK_MASK, CRT_UPCRT_LOCK) << 8) 
    | lockval2); /* Two locks to keep track of -- ick.*/
}

static inline int rlockCRT(void) 
{ 
  /* Timing registers don't have separate read lock -- yay. */
  return (outbitsCRT(CR29_RLOCK, CR29_RLOCK_MASK, CRT_UPCRT_LOCK));
}

static inline int runlockCRT(void) 
{ 
  /* Timing registers don't have separate read lock -- yay. */
  return (outbitsCRT(CR29_RUNLOCK, CR29_RLOCK_MASK, CRT_UPCRT_LOCK));
}


#endif /* _wd90c00_io_h */
