/* $Id: gwt_debug.h,v 1.2 1999/05/24 00:28:53 marcus Exp $
******************************************************************************

   LibGWT debugging macros

   Copyright (C) 1998	Marcus Sundberg		[marcus@ggi-project.org]
                 1999	Rodolphe Ortalo         [ortalo@laas.fr]
  
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#ifndef _GWT_INTERNAL_DEBUG_H
#define _GWT_INTERNAL_DEBUG_H

#include <stdio.h>
#include <stdarg.h>
#include <ggi/gg.h>
#include <ggi/ggi.h>

extern uint32 _gwtDebugState;
extern int _gwtDebugSync;

/* Debugging types
 * bit 0 is reserved! */

#define GWTDEBUG_CORE		(1<<1)	/*   2 */
#define GWTDEBUG_WINDOW		(1<<2)	/*   4 */
#define GWTDEBUG_REGION		(1<<3)	/*   8 */
#define GWTDEBUG_DRAW		(1<<4)	/*  16 */
#define GWTDEBUG_MISC		(1<<5)	/*  32 */
#define GWTDEBUG_EVENTS		(1<<6)	/*  64 */

#define GWTDEBUG_ALL	0xffffffff

#ifdef __GNUC__

#ifdef DEBUG
#define GWTDPRINT(form,args...)	    if (_gwtDebugState) { ggDPrintf(_gwtDebugSync,"LibGWT:",form, ##args); }
#define GWTDPRINT_CORE(form,args...)   if (_gwtDebugState & GWTDEBUG_CORE) { ggDPrintf(_gwtDebugSync,"LibGWT:",form, ##args); }
#define GWTDPRINT_WINDOW(form,args...)   if (_gwtDebugState & GWTDEBUG_WINDOW) { ggDPrintf(_gwtDebugSync,"LibGWT:",form, ##args); }
#define GWTDPRINT_REGION(form,args...)  if (_gwtDebugState & GWTDEBUG_REGION) { ggDPrintf(_gwtDebugSync,"LibGWT:",form, ##args); }
#define GWTDPRINT_DRAW(form,args...)   if (_gwtDebugState & GWTDEBUG_DRAW) { ggDPrintf(_gwtDebugSync,"LibGWT:",form, ##args); }
#define GWTDPRINT_MISC(form,args...)   if (_gwtDebugState & GWTDEBUG_MISC) { ggDPrintf(_gwtDebugSync,"LibGWT:",form, ##args); }
#define GWTDPRINT_EVENTS(form,args...) if (_gwtDebugState & GWTDEBUG_EVENTS) { ggDPrintf(_gwtDebugSync,"LibGWT:",form, ##args); }
#else /* DEBUG */
#define GWTDPRINT(form,args...)		do{}while(0)
#define GWTDPRINT_CORE(form,args...)	do{}while(0)
#define GWTDPRINT_WINDOW(form,args...)	do{}while(0)
#define GWTDPRINT_REGION(form,args...)	do{}while(0)
#define GWTDPRINT_DRAW(form,args...)	do{}while(0)
#define GWTDPRINT_MISC(form,args...)	do{}while(0)
#define GWTDPRINT_EVENTS(form,args...)	do{}while(0)
#endif /* DEBUG */

#else /* __GNUC__ */

static inline void GWTDPRINT(const char *form,...)
{
#ifdef DEBUG
	if (_gwtDebugState) {
		va_list args;

		fprintf(stderr, "LibGWT: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_gwtDebugSync) fflush(stderr);
	}
#endif
}

static inline void GWTDPRINT_CORE(const char *form,...)
{
#ifdef DEBUG
	if (_gwtDebugState & GWTDEBUG_CORE) {
		va_list args;

		fprintf(stderr, "LibGWT: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_gwtDebugSync) fflush(stderr);
	}
#endif
}

static inline void GWTDPRINT_WINDOW(const char *form,...)
{
#ifdef DEBUG
	if (_gwtDebugState & GWTDEBUG_WINDOW) {
		va_list args;

		fprintf(stderr, "LibGWT: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_gwtDebugSync) fflush(stderr);
	}
#endif
}

static inline void GWTDPRINT_REGION(const char *form,...)
{
#ifdef DEBUG
	if (_gwtDebugState & GWTDEBUG_REGION) {
		va_list args;

		fprintf(stderr, "LibGWT: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_gwtDebugSync) fflush(stderr);
	}
#endif
}

static inline void GWTDPRINT_DRAW(const char *form,...)
{
#ifdef DEBUG
	if (_gwtDebugState & GWTDEBUG_DRAW) {
		va_list args;

		fprintf(stderr, "LibGWT: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_gwtDebugSync) fflush(stderr);
	}
#endif
}

static inline void GWTDPRINT_MISC(const char *form,...)
{
#ifdef DEBUG
	if (_gwtDebugState & GWTDEBUG_MISC) {
		va_list args;

		fprintf(stderr, "LibGWT: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_gwtDebugSync) fflush(stderr);
	}
#endif
}

static inline void GWTDPRINT_EVENTS(const char *form,...)
{
#ifdef DEBUG
	if (_gwtDebugState & GWTDEBUG_EVENTS) {
		va_list args;

		fprintf(stderr, "LibGWT: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_gwtDebugSync) fflush(stderr);
	}
#endif
}

#endif /* __GNUC__ */

#ifdef DEBUG
#define LIBGWT_ASSERT(x,str) \
{ if (!(x)) { \
	fprintf(stderr,"LIBGWT:%s:%d: INTERNAL ERROR: %s\n",__FILE__,__LINE__,str); \
	exit(1); \
} }
#define LIBGWT_APPASSERT(x,str) \
{ if (!(x)) { \
	fprintf(stderr,"LIBGWT:%s:%d: APPLICATION ERROR: %s\n",__FILE__,__LINE__,str); \
	exit(1); \
} }
#else /* DEBUG */
#define LIBGWT_ASSERT(x,str)	do{}while(0)
#define LIBGWT_APPASSERT(x,str)	do{}while(0)
#endif /* DEBUG */

#endif /* _GWT_INTERNAL_DEBUG_H */
